#ifndef __cell_h_
#define __cell_h_
#include <adevs.h>

/**
Material parameters for van Genuchten material hydraulic conductivity and
moisture content models.  This class should be extended to provide appropriate
data.
*/
class MaterialParams
{
	public:
		/// Get the width (meters) of the solution space (0 is at the 'left')
		virtual double width() const = 0;
		/// Get the height (meters) of the solution space (0 is at the 'top')
		virtual double height() const = 0;
		/// Get the initial pressure
		virtual double h0(double x, double y) const = 0;
		/// Get the pressure flux
		virtual double qs(double x, double y) const = 0;
		/// Compute hydraulic conductivity
		virtual double K(double x, double y, double h) const = 0;
		/// Compute moisture capacitance (unsaturated) or specific storage (saturated)
		virtual double C_Ss(double x, double y, double h) const = 0;
		/// Compute moisture content
		virtual double theta(double x, double y, double h) const = 0;
		/// Returns true if material is saturated, false otherwise.
		virtual bool saturated(double x, double y, double h) const = 0;
		/// Destructor
		virtual ~MaterialParams()
		{
		}
};

/**
A model of a single cell in a 2D finite difference approximation of
the non-linear Richard's equation.
*/
class cell: public atomic, public cell_interface
{
	public:
		/**
		Cell input is pressure at neighboring cells.  Cell output
		is the pressure at this cell.
		*/
		/// Grid resolution
		static double ds;
		/// Quantum size
		static double D;

		/// Constructor
		cell(const MaterialParams* p, double x, double y);
		/// Copy constructor
		cell(const cell& src);
		/// Create a copy of this cell
		object* clone() const 
		{ 
			return new cell(*this); 
		}
		void init();
		void delta_int();
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		void delta_conf(const adevs_bag<PortValue>& x);
		void output_func(adevs_bag<PortValue>& y);
		void gc_output(adevs_bag<PortValue>& y){}
		double getState() const;
		/// Destructor
		~cell()
		{
		}
		/// Derivative function
		double dh(double hd);

	private:
		/// Location of the cell
		const double x, y;
		/// Integrator
		qoden2<1> h;
		/// Pressure at neighboring cells
		double hn[4];
		/// Moisture capacitance at this cells
		double C_Ss;
		/// Conductivity at cell boundaries
		double K[5];
		/// Cell material parameters
		const MaterialParams* p;
		/// Set neighboring cell pressure from input
		void scan_input(const adevs_bag<PortValue>& x);
		/// Boundary condition function
		void bc(double hd);

		// No public assignement operator
		void operator=(cell& src)
		{
		}
		// No default constructor
		cell::cell()
		{
		}
};

#endif
