/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999-2004 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __adevs_bag_h_
#define __adevs_bag_h_
#include "adevs_config.h"
#define BAG_DEFAULT_CAP 100
#define BAG_DEFAULT_GROW 100

ADEVS_NAMESPACE_BEGIN

/**
An bag stores a collection of elements.  It supports
indexed access to the elements (via the [] operator), addition of
new elements, and removal of elements.  The element type must support
the assignment and equality operators.

<p>New elements are appended to the end of the bag.
When an element is removed, the last element of the bag fills
the vacant slot, thereby avoiding holes in the indexing.
*/
template <class element_t> class adevs_bag
{
	public:		
		adevs_bag():
		cap(BAG_DEFAULT_CAP),
		grow(BAG_DEFAULT_GROW),
		size(0)
		{
			elements = new element_t[BAG_DEFAULT_CAP];
		}
		/**
		Capacity is the initial memory set aside to hold elements.
		When the adevs_bag becomes full, its capacity is increased by grow
		slots.
		*/
		adevs_bag(int capacity, int grow = BAG_DEFAULT_GROW):
		cap(capacity),
		grow(grow),
		size(0)
		{
			elements = new element_t[capacity];
		}
		/**
		Element copying is done via the assignment operator.
		*/
		adevs_bag(const adevs_bag<element_t>& src):
		cap(src.cap),
		grow(src.grow),
		size(src.size)
		{
			elements = new element_t[src.cap];
			for (int i = 0; i < size; i++) elements[i] = src.elements[i];
		}
		/**
		Element copying is done via the assignment operator.
		*/
		const adevs_bag<element_t>& operator=(const adevs_bag<element_t>& src)
		{
			cap = src.cap;
			grow = src.grow;
			size = src.size;
			delete [] elements;
			elements = new element_t[src.cap];
			for (int i = 0; i < size; i++) elements[i] = src.elements[i];
			return *this;
		}
		/// The last index is at size - 1
		int getSize() const { return size; }
		/// Reports the current capacity of the data structure
		int getCapacity() const { return cap; }
		/// Increases the capacity by adjustment value.
		void enlarge(int adjustment)
		{
			cap += adjustment;
			element_t* relements = new element_t[cap];
			for (int i = 0; i < size; i++) relements[i] = elements[i];
			delete [] elements;
			elements = relements;
		}
		/// Returns a reference to the object stored at index
		element_t& operator[](int index) 
		{
			return elements[index];
		}
		/// Returns a constant reference to the element stored at index
		const element_t& get(int index) const 
		{
			return elements[index];
		}
		/// Returns a reference to the element stored at index
		element_t& get(int index)
		{
			return elements[index];
		}
		/// Swaps the elements at index i and j
		void swap(int i, int j)
		{
			element_t tmp(elements[i]);
			elements[i] = elements[j];
			elements[j] = tmp;
		}
		/// Appends the items in the source adevs_bag to this adevs_bag.
		void append(const adevs_bag<element_t>& src)
		{
			int over_cap = src.getSize()-cap-size;
			if (over_cap > 0) enlarge(over_cap);
			for (int i = 0; i < src.getSize(); i++) add(src.get(i));
		}
		/// Removes all elements from the adevs_bag.
		virtual void removeAll() { size = 0; }
		/**
		Adds an element to the end of the adevs_bag (last index).
		*/
		virtual void add(const element_t& e)
		{
			if (cap == size) enlarge(grow);
			elements[size] = e;
			size++;
		}
		/// Removes the element at the indicated index.
		virtual void remove(int index)
		{
			size--;
			elements[index] = elements[size];
		}
		/// Find and erase the first element equal to e (as per the == operator).
		void erase(const element_t& e)
		{
			for (int i = 0; i < getSize(); i++)
			{
				if (get(i) == e)
				{
					remove(i);
					return;
				}
			}
		}
		virtual ~adevs_bag()
		{
			delete [] elements;
		}
	private:	
		int cap;
		int grow;
		int size;
		element_t* elements;
};

ADEVS_NAMESPACE_END

#endif
