/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2002, 2003 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __adevs_config_h_
#define __adevs_config_h_
#include <limits.h>
#include <float.h>
#include <cmath>
#include <typeinfo>

/******** COMPILE TIME CONFIGURATION ITEMS ******/

/**
This option defines a namespace for the simulation library.
Comment this line if you don't want to use a namespace.
*/
// #define USE_ADEVS_NAMESPACE adevs

// All configurable types need to have a default (no arg) constructor,
// assignment operator, and copy constructor.  These will exist outside
// of the namespace.

/**
Define the type for model inputs and outputs.
*/
typedef enum { BURNING, BURNT, UNBURNT } CellPhase;
#define ADEVS_IO_TYPE CellPhase

/**
Define the type used as model ports.  The port type
type must support the equality (== and !=) operators.
Note that this MUST be an unsigned long if you are going
to use the cellSpace model provided with the simulation
library.
*/
#define ADEVS_PORT_TYPE unsigned long
// Comment this line if the above is not unsigned long
#define OK_TO_COMPILE_ADEVS_CELLSPACE 

/**
Define the simulation clock type.  EPSILON is used as a time granule
for determined when a time t is 'equal' to a time t +- EPSILON.
Time should support operators sufficient to make it an Abelian
group using the + and - operators.  It should also
support ALL of the comparison (== != <= >= > <) operators
interpreted in the usual way.
*/ 
#define ADEVS_TIME_TYPE double
#define ADEVS_ZERO 0.0
#define ADEVS_EPSILON 1E-14
#define ADEVS_INFINITY DBL_MAX

/**
Support for parallel computing.
*/
// Define to use threads
// #define USE_THREADS
// Set to appropriate thread API
#define POSIX_THREADS

/******** END OF COMPILE TIME CONFIGURATION ITEMS ******/

// M$ Windows specific pragmas
#ifdef _WIN32
	// Disable some Windows-only warnings
	#pragma warning(disable: 4100 4512 4018 4245 4511 4663 4710 4702)
	// Sometimes these are defined as Windows macros
	#ifdef min
	#undef min
	#endif
	#ifdef max
	#undef max
	#endif
#endif

/// Redefines for backwards compatibility
#ifndef IO_TYPE
	#define IO_TYPE ADEVS_IO_TYPE
#endif
#ifndef PORT_TYPE
	#define PORT_TYPE ADEVS_PORT_TYPE
#endif
#ifndef TIME_TYPE
	#define TIME_TYPE ADEVS_TIME_TYPE
#endif
#ifndef INFINITY
	#define INFINITY ADEVS_INFINITY
#endif
#ifndef EPSILON 
	#define EPSILON ADEVS_EPSILON
#endif
#ifndef ZERO
	#define ZERO ADEVS_ZERO
#endif

// Setup namespace information
#ifdef USE_ADEVS_NAMESPACE
	#define ADEVS_NAMESPACE USE_ADEVS_NAMESPACE
	#define ADEVS_NAMESPACE_BEGIN namespace ADEVS_NAMESPACE {
	#define ADEVS_NAMESPACE_END }
	#define USING_NAMESPACE_ADEVS using namespace ADEVS_NAMESPACE;
#else
	#define ADEVS_NAMESPACE 
	#define ADEVS_NAMESPACE_BEGIN 
	#define ADEVS_NAMESPACE_END 
	#define USING_NAMESPACE_ADEVS
#endif

#endif
